package com.pl.projectionscreen;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.google.gson.Gson;
import com.hpplay.sdk.source.browse.api.LelinkServiceInfo;

import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.List;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * ProjectionscreenPlugin
 */
public class ProjectionscreenPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
    private String TAG = "ProjectionscreenPlugin";
    HashMap<String, LelinkServiceInfo> mDevicesMap = new HashMap<String, LelinkServiceInfo>();
    LelinkServiceInfo info;
    //事件派发
    private EventChannel.EventSink eventSink = null;
    private EventChannel.StreamHandler streamHandler = new EventChannel.StreamHandler() {
        @Override
        public void onListen(Object o, EventChannel.EventSink sink) {
            eventSink = sink;
        }

        @Override
        public void onCancel(Object o) {
            eventSink = null;
        }
    };

    private SDKManager mSDKManager;
    private boolean isFirstBrowse = true;
    //    private boolean isPause = false;
    private boolean isPlayMirror;
    private ProjectionscreenPlugin.UIHandler mDelayHandler;

    private class UIHandler extends Handler {

        private WeakReference<Activity> mReference;

        UIHandler(Activity reference) {
            mReference = new WeakReference<>(reference);
        }

        @Override
        public void handleMessage(Message msg) {
            Activity mainActivity = mReference.get();
            if (mainActivity == null) {
                return;
            }
            switch (msg.what) {
                case IUIUpdateListener.STATE_SEARCH_SUCCESS:
                    updateBrowseAdapter();
                    break;
            }
            super.handleMessage(msg);
        }
    }

    // SDK
    private LelinkHelper mLelinkHelper;
    private static final String WIFI_AP_STATE_CHANGED_ACTION = "android.net.wifi.WIFI_AP_STATE_CHANGED";
    private static final String NET_VIDEO_URL = "https://v.mifile.cn/b2c-mimall-media/ed921294fb62caf889d40502f5b38147.mp4";
    private static final String NET_MUSIC_URL = "http://music.163.com/song/media/outer/url?id=287248.mp3";
    private static final String NET_PICTURE_URL = "http://news.cri.cn/gb/mmsource/images/2013/06/23/2/2211679758122940818.jpg";
    private static final String LOCAL_MEDIA_PATH = "/hpplay_demo/local_media/";
    private static final int REQUEST_MUST_PERMISSION = 1;
    private static final int REQUEST_CAMERA_PERMISSION = 2;
    private static final int REQUEST_RECORD_AUDIO_PERMISSION = 4;


    public static final int TYPE_URL = 1;
    public static final int TYPE_MIRROR = 2;
    public static final int TYPE_SCREEN = 100;
    public static final int TYPE_AUDIO = 101;
    public static final int TYPE_VIDEO = 102;
    public static final int TYPE_IMAGE = 103;
    public static final int LOOP_MODE_UNDEFINED = -1;
    public static final int LOOP_MODE_DEFAULT = 0;
    public static final int LOOP_MODE_SINGLE = 1;
    public static final int LOOP_MODE_ALL = 2;
    public static final int MONITOR_START = 1;
    public static final int MONITOR_STOP = 2;
    public static final int MONITOR_PAUSE = 3;
    public static final int MONITOR_RESUME = 4;

    //    private static final String SDCARD_LOCAL_MEDIA_PATH = Environment.getExternalStorageDirectory()
    private ProjectionscreenPlugin.NetworkReceiver mNetworkReceiver;
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;
    private Activity context;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "projectionscreen");
        channel.setMethodCallHandler(this);
        new EventChannel(flutterPluginBinding.getBinaryMessenger(), "sample.flutter.io/test_event_channel").setStreamHandler(streamHandler);
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "projectionscreen");
        channel.setMethodCallHandler(new ProjectionscreenPlugin());
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        } else {
            result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    private void updateBrowseAdapter() {
        if (null != mLelinkHelper) {
            List<LelinkServiceInfo> infos = mLelinkHelper.getInfos();
            HashMap map = new HashMap<String, LelinkServiceInfo>();
            for (int i = 0; i < infos.size(); i++) {
                Log.e(TAG, "device ip :" + infos.get(i).getIp());
                map.put(infos.get(i).getIp(), infos.get(i));
            }
            mDevicesMap.clear();
            mDevicesMap.putAll(map);
            Gson gson = new Gson();
            String s = gson.toJson(infos);
//            Log.e(TAG, "browse result:" + s);
            if (eventSink != null) {
                HashMap map1 = new HashMap<String, String>();
                map1.put("devices", s);
                eventSink.success(map1);
            } else {
                Log.e(TAG, "error :eventSink not init");
            }

        }
    }

    //========================================================ActivityAware
    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }
    //========================================================

    private static class NetworkReceiver extends BroadcastReceiver {

        private WeakReference<Activity> mReference;

        public NetworkReceiver(Activity pReference) {
            mReference = new WeakReference<>(pReference);
        }

        @Override
        public void onReceive(Context context, Intent intent) {
            if (null == mReference || null == mReference.get()) {
                return;
            }
            Activity mainActivity = mReference.get();
            String action = intent.getAction();
            if (ConnectivityManager.CONNECTIVITY_ACTION.equalsIgnoreCase(action) ||
                    WIFI_AP_STATE_CHANGED_ACTION.equalsIgnoreCase(action)) {
//                mainActivity.refreshWifiName();
            }
        }
    }

}
